﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using System.Windows.Input;
using System.Collections.ObjectModel;
using ArcGIS.Desktop.Mapping;
using System.Windows.Data;
using ArcGIS.Desktop.Framework.Threading.Tasks;

namespace HookingCommands
{
  internal class MvvmDockPaneViewModel : DockPane
  {
    private const string _dockPaneID = "Mvvm_MvvmDockPane";
    /// <summary>
    /// used to lock collections for use by multiple threads
    /// </summary>
    private readonly object _lockCollection = new object();
    // Read only alias for map bookmarks.
    private readonly ReadOnlyObservableCollection<Bookmark> _readOnlyBookmarks;
    private readonly ObservableCollection<Bookmark> _bookmarks = new ObservableCollection<Bookmark>();

    private Bookmark _selectedBmk;

    protected MvvmDockPaneViewModel()
    {
      _readOnlyBookmarks = new ReadOnlyObservableCollection<Bookmark>(_bookmarks);
      BindingOperations.EnableCollectionSynchronization(_readOnlyBookmarks,
                        _lockCollection);
    }

    /// <summary>
    /// Show the DockPane.
    /// </summary>
    internal static void Show()
    {
      DockPane pane = FrameworkApplication.DockPaneManager.Find(_dockPaneID);
      if (pane == null)
        return;

      pane.Activate();
    }

    #region Buttons
    public ICommand RefreshBookMarkCommand
    {
      get
      {
        return new RelayCommand(() =>
       {
         var map = MapView.Active?.Map;
         if (map == null) return;
           // no need to await
          UpdateBookmarks(map);
         }, true);
      }
    }
    
    private async Task UpdateBookmarks(Map selectedMap)
    {
      lock (_lockCollection)
      {
        _bookmarks.Clear();
      }
      // get the bookmarks.  GetBookmarks needs to be on MCT but want to refresh members and properties on UI thread
      await QueuedTask.Run(() =>
      {
        foreach (var bmk in selectedMap.GetBookmarks())
        {
          lock (_lockCollection)  _bookmarks.Add(bmk);
        }
      });
    }
    
    #endregion // Buttons

    #region Bookmarks

    /// <summary>
    /// Holds the selected bookmark from the listview. 
    /// </summary>
    public Bookmark SelectedBookmark
    {
      get { return _selectedBmk; }
      set
      {
        SetProperty(ref _selectedBmk, value, () => SelectedBookmark);
        // zoom to it
        if (MapView.Active != null && SelectedBookmark != null) MapView.Active.ZoomToAsync(SelectedBookmark);
      }
    }

    /// <summary>
    /// collection of bookmarks.  Bind to this property in the view.
    /// </summary>
    public ReadOnlyObservableCollection<Bookmark> Bookmarks => _readOnlyBookmarks;

    #endregion Properties

    /// <summary>
    /// Text shown near the top of the DockPane.
    /// </summary>
    private string _heading = "MvvM Demo Dockpane";
    public string Heading
    {
      get { return _heading; }
      set
      {
        SetProperty(ref _heading, value, () => Heading);
      }
    }
  }

  /// <summary>
  /// Button implementation to show the DockPane.
  /// </summary>
  internal class MvvmDockPane_ShowButton : Button
  {
    protected override void OnClick()
    {
      MvvmDockPaneViewModel.Show();
    }
  }
}
